@@ -63,7 +63,7 @@ class Projection(collections.abc.Mapping):
         return len(tuple(iter(self)))
 
 
-class DictFilter(object):
+class DictFilter(collections.abc.Mapping):
     """
     Takes a dict, and simulates a sub-dict based on the keys.
 
@@ -92,15 +92,21 @@ class DictFilter(object):
     ...
     KeyError: 'e'
 
+    >>> 'e' in filtered
+    False
+
+    Pattern is useful for excluding keys with a prefix.
+
+    >>> filtered = DictFilter(sample, include_pattern=r'(?![ace])')
+    >>> dict(filtered)
+    {'b': 2, 'd': 4}
+
     Also note that DictFilter keeps a reference to the original dict, so
     if you modify the original dict, that could modify the filtered dict.
 
     >>> del sample['d']
-    >>> del sample['a']
-    >>> filtered == {'b': 2, 'c': 3}
-    True
-    >>> filtered != {'b': 2, 'c': 3}
-    False
+    >>> dict(filtered)
+    {'b': 2}
     """
 
     def __init__(self, dict, include_keys=[], include_pattern=None):
@@ -120,29 +126,18 @@ class DictFilter(object):
 
     @property
     def include_keys(self):
-        return self.specified_keys.union(self.pattern_keys)
-
-    def keys(self):
-        return self.include_keys.intersection(self.dict.keys())
-
-    def values(self):
-        return map(self.dict.get, self.keys())
+        return self.specified_keys | self.pattern_keys
 
     def __getitem__(self, i):
         if i not in self.include_keys:
             raise KeyError(i)
         return self.dict[i]
 
-    def items(self):
-        keys = self.keys()
-        values = map(self.dict.get, keys)
-        return zip(keys, values)
+    def __iter__(self):
+        return filter(self.include_keys.__contains__, self.dict.keys())
 
-    def __eq__(self, other):
-        return dict(self) == other
-
-    def __ne__(self, other):
-        return dict(self) != other
+    def __len__(self):
+        return len(list(self))
 
 
 def dict_map(function, dictionary):
@@ -370,7 +365,7 @@ class FoldedCaseKeyedDict(KeyTransformingDict):
     True
     >>> 'HELLO' in d
     True
-    >>> print(repr(FoldedCaseKeyedDict({'heLlo': 'world'})).replace("u'", "'"))
+    >>> print(repr(FoldedCaseKeyedDict({'heLlo': 'world'})))
     {'heLlo': 'world'}
     >>> d = FoldedCaseKeyedDict({'heLlo': 'world'})
     >>> print(d['hello'])
@@ -573,6 +568,8 @@ class DictStack(list, collections.abc.Mapping):
     2
     >>> stack['c']
     2
+    >>> len(stack)
+    3
     >>> stack.push(dict(a=3))
     >>> stack['a']
     3
@@ -598,7 +595,7 @@ class DictStack(list, collections.abc.Mapping):
         return iter(set(itertools.chain.from_iterable(c.keys() for c in dicts)))
 
     def __getitem__(self, key):
-        for scope in reversed(self):
+        for scope in reversed(tuple(list.__iter__(self))):
             if key in scope:
                 return scope[key]
         raise KeyError(key)
@@ -608,6 +605,9 @@ class DictStack(list, collections.abc.Mapping):
     def __contains__(self, other):
         return collections.abc.Mapping.__contains__(self, other)
 
+    def __len__(self):
+        return len(list(iter(self)))
+
 
 class BijectiveMap(dict):
     """
